@@ -158,6 +158,7 @@ typedef enum
   STATE_MONITOR_MODE_RATE,
   STATE_MONITOR_MODE_FLAG,
   STATE_MONITOR_UNDERSCANNING,
+  STATE_MONITOR_MAXBPC,
   STATE_DISABLED,
   STATE_POLICY,
   STATE_STORES,
@@ -497,6 +498,10 @@ handle_start_element (GMarkupParseContext  *context,
             {
               parser->state = STATE_MONITOR_UNDERSCANNING;
             }
+          else if (g_str_equal (element_name, "maxbpc"))
+            {
+              parser->state = STATE_MONITOR_MAXBPC;
+            }
           else
             {
               g_set_error (error, G_MARKUP_ERROR, G_MARKUP_ERROR_UNKNOWN_ELEMENT,
@@ -590,6 +595,13 @@ handle_start_element (GMarkupParseContext  *context,
           return;
         }
 
+      case STATE_MONITOR_MAXBPC:
+        {
+          g_set_error (error, G_MARKUP_ERROR, G_MARKUP_ERROR_UNKNOWN_ELEMENT,
+                       "Invalid element '%s' under maxbpc", element_name);
+          return;
+        }
+
       case STATE_DISABLED:
         {
           if (!g_str_equal (element_name, "monitorspec"))
@@ -791,6 +803,7 @@ finish_monitor_spec (ConfigParser *parser)
       case STATE_MONITOR_MODE_RATE:
       case STATE_MONITOR_MODE_FLAG:
       case STATE_MONITOR_UNDERSCANNING:
+      case STATE_MONITOR_MAXBPC:
       case STATE_POLICY:
       case STATE_STORES:
       case STATE_STORE:
@@ -899,6 +912,14 @@ handle_end_element (GMarkupParseContext  *context,
           return;
         }
 
+      case STATE_MONITOR_MAXBPC:
+        {
+          g_assert (g_str_equal (element_name, "maxbpc"));
+
+          parser->state = STATE_MONITOR;
+          return;
+        }
+
       case STATE_MONITOR:
         {
           GfLogicalMonitorConfig *logical_monitor_config;
@@ -1378,6 +1399,29 @@ handle_text (GMarkupParseContext  *context,
           return;
         }
 
+      case STATE_MONITOR_MAXBPC:
+        {
+          int signed_max_bpc;
+
+          if (read_int (text, text_len, &signed_max_bpc, error))
+            {
+              if (signed_max_bpc >= 0)
+                {
+                  parser->current_monitor_config->has_max_bpc = TRUE;
+                  parser->current_monitor_config->max_bpc = signed_max_bpc;
+                }
+              else
+                {
+                  g_set_error (error, G_MARKUP_ERROR,
+                               G_MARKUP_ERROR_INVALID_CONTENT,
+                               "Invalid negative maxbpc value '%s'",
+                               text);
+                }
+            }
+
+          return;
+        }
+
       case STATE_STORE:
         {
           GfConfigStore store;
@@ -1602,12 +1646,15 @@ append_monitors (GString *buffer,
     {
       GfMonitorConfig *monitor_config;
       GfMonitorModeSpec *mode_spec;
-      gchar rate_str[G_ASCII_DTOSTR_BUF_SIZE];
+      char rate_str[G_ASCII_DTOSTR_BUF_SIZE];
 
       monitor_config = l->data;
       mode_spec = monitor_config->mode_spec;
 
-      g_ascii_dtostr (rate_str, sizeof (rate_str), mode_spec->refresh_rate);
+      g_ascii_formatd (rate_str,
+                       sizeof (rate_str),
+                       "%.3f",
+                       mode_spec->refresh_rate);
 
       g_string_append (buffer, "      <monitor>\n");
       append_monitor_spec (buffer, monitor_config->monitor_spec, "        ");
@@ -1620,6 +1667,13 @@ append_monitors (GString *buffer,
       g_string_append (buffer, "        </mode>\n");
       if (monitor_config->enable_underscanning)
         g_string_append (buffer, "        <underscanning>yes</underscanning>\n");
+
+      if (monitor_config->has_max_bpc)
+        {
+          g_string_append_printf (buffer, "        <maxbpc>%u</maxbpc>\n",
+                                  monitor_config->max_bpc);
+        }
+
       g_string_append (buffer, "      </monitor>\n");
     }
 }
